
#ifndef LOCAL_CONFIG_H
#define LOCAL_CONFIG_H

#undef HAVE_TIMEGM
#define HAVE_TIMEGM 1

#undef HAVE_LOCALTIME_R
#define HAVE_LOCALTIME_R 1

#undef HAVE_GMTIME_R
#define HAVE_GMTIME_R 1

#undef HAVE_ZLIB
#define HAVE_ZLIB 1

#undef HAVE_OPENSSL
#define HAVE_OPENSSL 1

#undef CC_INLINE
#define CC_INLINE inline

#undef NSROOT
#define NSROOT SONOS

#undef LIBTAG
#define LIBTAG "SONOS"

#undef LIBVERSION
#define LIBVERSION "2.12.11"

#undef LIBVERSION_MAJOR
#define LIBVERSION_MAJOR 2

#undef LIBVERSION_MINOR
#define LIBVERSION_MINOR 12

#undef LIBVERSION_PATCH
#define LIBVERSION_PATCH 11

#undef HAVE_PULSEAUDIO
/* #undef HAVE_PULSEAUDIO */

#undef HAVE_FLAC
#define HAVE_FLAC 1

#undef HAVE_MP3LAME
/* #undef HAVE_MP3LAME */

#endif	/* LOCAL_CONFIG_H */
